****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_squaredtableshell.sas  
*
* Created (mm/dd/yyyy): 06/01/2018
* Last modified: 02/07/2019
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This macro creates a "squared" stratification table                                    
*   
*  Program inputs:                                                                                            
* 
*  Program outputs:                                                                                                                                       
* 
*  PARAMETERS:                                                                       
*
* 
*  Programming Notes:                                                                                
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*	1.1		  01/07/19	  AP		Added squared gapnum
*
*	1.1		  02/07/19	  AP		Added squared episodenum
*
***************************************************************************************************;

%macro ms_squaredtableshell(squarevars = , analysisvars=, groups=, analysis=);

%put =====> MACRO CALLED: ms_squaredtableshell v1.2;

	%global covarstrat;
	%let covarstrat = ;

	%if %str(&squarevars.) ne %str() %then %do;

		*Base dataset - square AgeGroup, Sex, Year, Month;
		%if &analysis. = N %then %do;
		data _mastersquare(rename=cohortgrp=group);
			set cohortfile(
				%if %str(&groups.) ne () %then %do; where=(strip(cohortgrp) in (&groups.))%end; keep=cohortgrp);
		%end;
		%if &analysis. = Y %then %do;
		data _mastersquare;
			length analysisgrp $40;
			analysisgrp = &groups.;
		%end;
			%if %index(&squarevars., sex) > 0 %then %do;
			format sex $1.;	
			%end;

			%if %str(&analysisvars.) ne %str() %then %do;
			%do a = 1 %to %sysfunc(countw(&analysisvars.));
				%scan(&analysisvars., &a.) = 0;
			%end;
			%end;

			%if %index(&squarevars., agegroup) > 0 %then %do;
			    format AgeGroup $9.;
				numgrp = countw("&AGESTRAT."," ") ;

			    do i=1 to numgrp;
			        AgeGroup=scan("&AGESTRAT",i," ");
			        AgeGroupNum=i;
			%end;

			*Either square month/year or year only;
			%if %index(&squarevars., month) > 0 %then %do;
			    do per=1 to &nummonths.;
			     	*create year month variables;
			        date=intnx('month',&startfollowup.,per-1,'begin');
			        month=month(date);
			        year=year(date);
						%if %index(&squarevars., sex) > 0 %then %do;
				            sex="F"; output;
				            sex="M"; output;
				            sex="O"; output;
						%end;
						%else %do;
							output;
						%end;
				end;
			%end;
			%else %if %index(&squarevars., year) > 0 %then %do;
		 		do per=1 to &numyears.;
			    	*create year variables;
			    	date=intnx('year',&startfollowup.,per-1,'begin');
			        year=year(date);
						%if %index(&squarevars., sex) > 0 %then %do;
				            sex="F"; output;
				            sex="M"; output;
				            sex="O"; output;
						%end;
						%else %do;
							output;
						%end;
			 	end;
			%end;
			%else %if %index(&squarevars., sex) > 0 %then %do;
			  		sex="F"; output;
			        sex="M"; output;
					sex="O"; output;
			%end;
			%else %if %index(&squarevars., agegroup) > 0 %then %do;
				output;
			%end;

			%if %index(&squarevars., agegroup) > 0 %then %do;
			 end;
			%end;


			keep %if &analysis = N %then %do; cohortgrp %end;
				 %if &analysis = Y %then %do; analysisgrp %end;
				&analysisvars.
				%if %index(&squarevars., agegroup) > 0 %then %do; agegroup agegroupnum %end;
				%if %index(&squarevars., sex) > 0 %then %do; sex %end;


				%if %index(&squarevars., month) > 0 %then %do; month year %end;
				%else %if %index(&squarevars., year) > 0 %then %do; year %end;
			;
		run;

		*Type 3 specific stratifiers: TTC and TTE;
		%if %eval(&type.=3) %then %do;
			%if %index(&squarevars., ttc) > 0 | %index(&squarevars., tte) > 0 %then %do;
			
				*Determine min --> max windows;
				data _null_;
					minfrom = min(&T3CtrlFrom., &T3RiskFrom.);
					maxto = max(&T3CtrlTo., &T3RiskTo.);
					call symputx("minwinfrom", minfrom);
					call symputx("maxwinto", maxto);
				run;

				data _tte_ttc;
					do TTE_VALUE=&minwinfrom. to &maxwinto.;
						TTC_VALUE=TTE_VALUE;
						output;
			        end;	       
			    run;

				proc sql noprint undo_policy=none;
					create table _mastersquare as
					select base.*,
				           com.tte_value,
						   com.ttc_value
				    from _mastersquare as base,
				       	 _tte_ttc as com;
				quit;
			%end;
		%end;

		*Type 4: specific stratifiers: PrePostInd, PregDur Codes, MP#s;
		%if %eval(&type.=4) %then %do;
			%if %index(&squarevars., prepostind) > 0 %then %do;
				data _prepostind; 
					format PrePostInd $10.;
						PrePostInd="TERM";	
						output;
						PrePostInd="PRETERM";	
						output;
						PrePostInd="POSTTERM";
						output;		
						PrePostInd="NONE";
						output;
				run;

				proc sql noprint undo_policy=none;
					create table _mastersquare as
					select base.*,
				           com.PrePostInd
				    from _mastersquare as base,
				       	 _prepostind as com;
				quit;
			%end;
			%if %index(&squarevars., pregdurcode) > 0 %then %do;
				proc sort nodupkey data=infolder.&PREGDUR.(where=(PriorityGroup1=1 or PriorityGroup2=1)) 
					out= _prePostTerm(keep=CodeCat CodeType Code duration);
				    by Code;
				run; 

				data _prePostTerm;
					set _prePostTerm;
					Code=compress(Code,'. ');
					format PrePostInd $10.;
					if Duration <=258 then do; PrePostInd="PRETERM"; output; end;
					if 259 <= Duration <=280 then do; PrePostInd="TERM"; output; end;
					if 281 <= Duration then do; PrePostInd="POSTTERM"; output; end;
				run;

				%ISDATA(dataset=_prePostTerm);
				%IF %EVAL(&NOBS.>=1) %THEN %DO;
					%MS_ProcessWildcards(InFile=_prePostTerm, 
					                     CodeVar=code, 
					                     OutFile=_prePostTerm);
				%END;

				proc sql noprint undo_policy=none;
					create table _mastersquare as
					select sq1.*,
						   sq2.Code as PregDurCode
					from _mastersquare as sq1 
						 left join _prePostTerm as sq2
						 on sq1.prepostind = sq2.prepostind;
				quit;
			%end;
			%if %index(&squarevars., moiname) > 0 %then %do;
				proc sort nodupkey data=infolder.&COHORTCODES. out=_MPsSquared(keep=T4_INDEX group rename=T4_INDEX=MOIName);
					by group T4_INDEX;
					where T4_INDEX =:"MP" and strip(group) = "&itgroup";
				run;

				data _MPsSquared;
					set _MPsSquared;
					by group MoiName;
					output;
					if last.group then do;		
						MoiName="";
						output;
					end;
				run;	

				proc sql noprint undo_policy=none;
					create table _mastersquare as
				    select base.*,
				          com.MOIName
				    from  _mastersquare as base,
				          _MPsSquared as com;
				quit;
			%end;
		%end;

		*Additional potential stratifiers:
			- Race
			- Hispanic
			- Covar#
			- Geographic variables;

		%if %index(&squarevars., race) > 0 %then %do;
			data _race;
				format race $1.;
				do i=0 to 5;
					race=i;
				    output;
				end;
			run;

			proc sql noprint undo_policy=none;
				create table _mastersquare as
				select base.*,
			           com.race
			    from _mastersquare as base,
			       	 _race as com;
			quit;
		%end;

		%if %index(&squarevars., hispanic) > 0 %then %do;
			data _hispanic;
				format hispanic $1.;
				hispanic="N";
				output;
				hispanic="Y";
				output;
				hispanic="U";
				output;
			run;

			proc sql noprint undo_policy=none;
				create table _mastersquare as
			    select base.*,
			           com.hispanic
			    from _mastersquare as base,
			       	 _hispanic as com;
			quit;
		%end;

		*Covariate stratifications;
		%if %index(&squarevars., covar) > 0 %then %do;
			%macro wrapper();
				*put covariates into macro var;
				%do c = 1 %to %sysfunc(countw(&squarevars.));
					%if %index(%scan(&squarevars., &c.), covar) > 0 %then %do;
						%let covarstrat = &covarstrat. %scan(&squarevars., &c.);
					%end;
				%end;

				%do cov = 1 %to %sysfunc(countw(&covarstrat.));		
					data _%scan(&covarstrat., &cov.);
						%scan(&covarstrat., &cov.) = 0;
						output;
						%scan(&covarstrat., &cov.) = 1;
						output;
					run;
				%end;

				proc sql noprint undo_policy=none;
					create table _covar as
					select 
						%do cov = 1 %to %sysfunc(countw(&covarstrat.));
						%if %eval(&cov) ne 1 %then %do;, %end; x&cov..%scan(&covarstrat., &cov.)
						%end;
					from 
						%do cov = 1 %to %sysfunc(countw(&covarstrat.));
						 %if %eval(&cov) ne 1 %then %do;, %end; _%scan(&covarstrat., &cov.) as x&cov.
						%end;
					;
				quit;

				proc sql noprint undo_policy=none;
				   create table _mastersquare as
				   select base.*,
				          com.*
				   from _mastersquare as base,
				        _covar as com;
				quit;
			%mend;
			%wrapper();
		%end;

		%if "&geog_out." = "Y" %then %do;	
			%if %index(&squarevars., zip_uncertain) > 0 %then %do;
			data _uncertain;
				zip_uncertain='N'; output;
				zip_uncertain='Y'; output;
			run;	

			proc sql noprint undo_policy=none;
			   	create table _mastersquare as
				select base.*,
					   com.zip_uncertain
				from _mastersquare as base,
					 _uncertain as com;
			quit;
			%end; 

			%macro squaregeog(var=, select=);
				proc sql noprint;
					create table _geog&var. as
					select distinct &select. format=$7. length=7
					from infolder.&zipfile.;
				quit;

				data _geog&var.;
					set _geog&var. end=eof;
					format &var. $7.;
					output;
					if eof then do;
				       	&var.='Missing';
					 	output;
						&var.='Invalid';
						output;
						%if "&var" = "hhs_reg" | "&var" = "cb_reg" %then %do;
						&var.='Other';
						output;
						%end;
					end;
				run;
	
				proc sql noprint undo_policy=none;
					create table _mastersquare as
					select base.*,
					       com.&var.
					from  _mastersquare as base,
			              _geog&var. as com;
			    quit;

			%mend;

			%if %index(%lowcase(&squarevars.), zip3) > 0 %then %do;
				%squaregeog(var=zip3, select = substr(zip,1,3) as zip3);
			%end;
			%if %index(%lowcase(&squarevars.), state) > 0 %then %do;
				%squaregeog(var=state, select = statecode as state);
			%end;
			%if %index(%lowcase(&squarevars.), cb_reg) > 0 %then %do;
				%squaregeog(var=cb_reg, select = cb_region as cb_reg);
			%end;
			%if %index(%lowcase(&squarevars.), hhs_reg) > 0 %then %do;
				%squaregeog(var=hhs_reg, select = hhs_region as hhs_reg);
			%end;

		%end; /*end geog*/

		*Gapnum - only square values of 1, 2, and 999;
		%if %index(&squarevars., gapnum) > 0 %then %do;
			data _gapnum;
				length gapnum 4;
				gapnum = 1;
				output;
				gapnum = 2;
				output;
				gapnum = 999;
				output;
			run;

			proc sql noprint undo_policy=none;
				create table _mastersquare as
				select base.*,
			           com.gapnum
			    from _mastersquare as base,
			       	 _gapnum as com;
			quit;
		%end;

		*Episodenum - square values of 1 and 2;
		%if %index(&squarevars., episodenum) > 0 %then %do;
			data _episodenum;
				length episodenum 4;
				episodenum = 1;
				output;
				episodenum = 2;
				output;
			run;

			proc sql noprint undo_policy=none;
				create table _mastersquare as
				select base.*,
			           com.episodenum
			    from _mastersquare as base,
			       	 _episodenum as com;
			quit;
		%end;


		%if %eval(&type.=6) & &analysis=Y %then %do;
			*Switching - TTS (time to switch);
			%if %index(&squarevars., ttswitch) > 0 %then %do;
				proc sql noprint undo_policy=none;
					create table _mastersquare as
					select x.*, y.*
					from _mastersquare as x,
					_TTS_squared as y;
				quit;
			%end;

			*Switching - EpisodeLength;
			%if %index(&squarevars., episodelength) > 0 %then %do;
				proc sql noprint undo_policy=none;
					create table _mastersquare as
					select x.*, y.*
					from _mastersquare as x,
					_length_squared as y;
				quit;
			%end;	
		%end;

		*Square TTE_CAT, ADHERENCE, ADHERENCE_#, EPIGAP;
		%if "&analysis" = "Y" %then %do;
			%if %index(&squarevars., tte_cat) > 0 %then %do;
				data _tte_cat(rename=tmp_ttecat=tte_cat);
					length tmp_ttecat $20;
					set secondaryinputfile(keep=analysisgrp tte_output_cat);
						where analysisgrp in (&groups.);

						count=countw(tte_output_cat, ' ');
						do i = 1 to count;
							tmp_ttecat = scan(tte_output_cat, i, ' ');
							ttecatnum = i;
							output;
						end;
						
						tmp_ttecat = 'Other';
						ttecatnum = count+1;
						output;
						drop i tte_output_cat count analysisgrp;
				run;

				proc sql noprint undo_policy=none;
					create table _mastersquare as
					select x.*, y.*
					from _mastersquare as x,
					_tte_cat as y;
				quit;
			%end;

			%if %index(&squarevars., adherence) > 0 & %index(&squarevars., adherence_) = 0 %then %do;
				data _adherence;
					adherence = 0;
					output;
					adherence = 1;
					output;
				run;

				proc sql noprint undo_policy=none;
					create table _mastersquare as
					select x.*, y.*
					from _mastersquare as x,
					_adherence as y;
				quit;
			%end;

			%if %index(&squarevars., adherence_) > 0 %then %do;
				proc sql noprint;
					select count(*) into: num_adherence
					from adherencefile
					where upcase(ANALYSISGRP) = upcase("&ANALYSISGRP.");
				quit;

				%do ad = 1 %to %eval(&num_adherence.);
					data _adherence_&ad.;
						adherence_&ad. = 0;
						output;
						adherence_&ad. = 1;
						output;
					run;

					proc sql noprint undo_policy=none;
						create table _mastersquare as
						select x.*, y.*
						from _mastersquare as x,
						_adherence_&ad. as y;
					quit;
				%end;
			%end;
			%if %index(&squarevars., epi_gap) > 0 %then %do;
				proc sql noprint;
					create table _epigap as
					select distinct epi_gap from dplocal.&runid._mstr_epigap;
				quit;

				proc sql noprint undo_policy=none;
					create table _mastersquare as
					select x.*, y.*
					from _mastersquare as x,
					_epigap as y;
				quit;
			%end;
		%end;

		*Delete datasets;
		proc datasets nowarn noprint lib=work;
			delete _tte: _prepostind _prePostTerm _square: _race _hispanic _cov: _uncertain _geog _adhere: _epigap;
		quit;

	%end;
	%else %do;
		%if &analysis. = N %then %do;
		data _mastersquare(rename=cohortgrp=group);
			set cohortfile(
				%if %str(&groups.) ne () %then %do; where=(strip(cohortgrp) in (&groups.))%end; keep=cohortgrp);
		%end;
		%if &analysis. = Y %then %do;
		data _mastersquare;
			length analysisgrp $40;
			analysisgrp = &groups.;
		%end;
		 	%if %str(&analysisvars.) ne %str() %then %do;
			%do a = 1 %to %sysfunc(countw(&analysisvars.));
				%scan(&analysisvars., &a.) = 0;
			%end;
			%end;
		 run;
	%end;

%put NOTE: ******** END OF MACRO: ms_squaredtableshell v1.2 ********;

%mend ms_squaredtableshell;
